#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>
#include "7z.h"

void usage(char *strName)
{
    printf("Usage:  %s inputFile outputFile\n", strName);
}

int main (int argc, char **argv)
{
    char inputFile[256], outputFile[256];
    FILE *hInputFile = NULL, *hOutputFile = NULL;
    struct stat s;
    unsigned char *inData, *outData;
    unsigned long inLen, outLen;
    unsigned lzma_algo;
    unsigned lzma_dictsize;
    unsigned lzma_fastbytes;
    bool bRet;

    if (argc != 3) {
	usage(argv[0]);
	exit (-1);
    }
    
    strcpy(inputFile, argv[1]);
    strcpy(outputFile, argv[2]);

    if (stat(inputFile, &s ) != 0) {
        printf( "Error opening input file %s\n", inputFile);
        return -1;
    }

    inLen = s.st_size;
    outLen = 23 * (unsigned long) pow(2, 20);

    hInputFile = fopen(inputFile, "rb");
    if (hInputFile ==  NULL) {
        printf( "Error opening input file %s\n", inputFile);
        return -1;
    }

    inData = (unsigned char *) malloc(inLen);
    outData = (unsigned char *) malloc(inLen);

    if ((inData == NULL) || (outData == NULL)) {
    	printf("Error allocating memory\n");
        fclose(hInputFile);
        return -3;
    }

    if (fread(inData, sizeof(char), inLen, hInputFile) != inLen) {
	printf("Error reading input file %s\n", inputFile);
        fclose(hInputFile);
        return -5;
    }

    fclose(hInputFile);
    
    lzma_algo = 2;
    lzma_dictsize = 1 << 22;
    lzma_fastbytes = 128;

    bRet = compress_lzma_7z((const unsigned char*) inData, 
        (unsigned) inLen, 
        (unsigned char*) outData, 
        (unsigned &) outLen, 
        lzma_algo, 
        lzma_dictsize,
        lzma_fastbytes);

    if (!bRet) {
        printf("Error compressing lzma\n");
        return -6;
    }
    
    hOutputFile = fopen(outputFile, "w+" );
    if (hOutputFile == NULL) {
        printf ("Error opening output file %s\n", outputFile);
        return -7;
    }

    fwrite(outData, outLen, 1, hOutputFile);

    fclose(hOutputFile);

    printf("SUCCESS\n");

    if (inData) {
        free(inData);
    }
    if (outData) {
        free(outData);
    }

    return(0);
}

